/*
 * Copyright (c) 2005, Joshua Wright <jwright@hasborg.com>
 *
 * $Id: bluecrypt_t.c,v 1.5 2005/09/07 22:50:17 jwright Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation. See COPYING for more
 * details.
 *
 * This code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <stdint.h>
#include "bluecrypt.h"
#include "utils.h"
#include "tomcrypt.h"



int test_e21()
{

	/* testing data taken from Bluetooth 2.0 specification */
	int testcount = 4, i, errorcount=0;
	uint8_t rand[4][16] = {
		{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /* Test 1 */
		 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},
		{0x2d, 0xd9, 0xa5, 0x50, 0x34, 0x31, 0x91, 0x30, /* Test 2 */
		 0x40, 0x13, 0xb2, 0xd7, 0xe1, 0x18, 0x9d, 0x09},
		{0xda, 0xb3, 0xcf, 0xfe, 0x9d, 0x57, 0x39, 0xd1, /* Test 3 */
		 0xb7, 0xbf, 0x4a, 0x66, 0x7a, 0xe5, 0xee, 0x24},
		{0x13, 0xec, 0xad, 0x08, 0xad, 0x63, 0xc3, 0x7f, /* Test 4 */
		 0x8a, 0x54, 0xdc, 0x56, 0xe8, 0x2f, 0x4d, 0xc1}
	};

	uint8_t bdaddr[4][6] = {
		{0x00, 0x00, 0x00, 0x00, 0x00, 0x00},
		{0xca, 0xc4, 0x36, 0x43, 0x03, 0xb6}, /* Test 2 */
		{0x02, 0xf8, 0xfd, 0x4c, 0xd6, 0x61}, /* Test 3 */
		{0x98, 0x46, 0xc5, 0xea, 0xd4, 0xd9}  /* Test 4 */
	};

	uint8_t ka[4][16] = {
		{0xd1, 0x4c, 0xa0, 0x28, 0x54, 0x5e, 0xc2, 0x62, /* Test 1 */
		 0xce, 0xe7, 0x00, 0xe3, 0x9b, 0x5c, 0x39, 0xee},
		{0xe6, 0x2f, 0x8b, 0xac, 0x60, 0x91, 0x39, 0xb3, /* Test 2 */
		 0x99, 0x9a, 0xed, 0xbc, 0x9d, 0x22, 0x80, 0x42},
		{0xb0, 0x37, 0x6d, 0x0a, 0x9b, 0x33, 0x8c, 0x2e, /* Test 3 */
		 0x13, 0x3c, 0x32, 0xb6, 0x9c, 0xb8, 0x16, 0xb3},
		{0x5b, 0x61, 0xe8, 0x3a, 0xd0, 0x4d, 0x23, 0xe9, /* Test 4 */
		 0xd1, 0xc6, 0x98, 0x85, 0x1f, 0xa3, 0x04, 0x47}
	};

	uint8_t linkkey[16];

	for (i = 0; i < testcount; i++) {

		printf("Testing E21 case %d of %d ... ", i + 1, testcount);

		memset(&linkkey, 0, sizeof(linkkey));
		if (e21(rand[i], bdaddr[i], linkkey) != 0) {
			printf("Error from e21(), ");
		}

		if (memcmp(linkkey, ka[i], 16) == 0) {
			printf("successful.\n");
		} else {
			printf("failed.\n");
			errorcount++;
		}
	}

	return errorcount;

}

int test_kmaster()
{
	int i, testcount = 3; int errorcount=0;
	uint8_t out_kmaster[16];

	uint8_t rand[3][16] = {
		{0x00, 0x1d, 0xe1, 0x69, 0x24, 0x88, 0x50, 0x24, 
		 0x5a, 0x5f, 0x7c, 0xc7, 0xf0, 0xd6, 0xd6, 0x33}, /* Test 1 */
		{0x67, 0xed, 0x56, 0xbf, 0xcf, 0x99, 0x82, 0x5f, 
		 0x0c, 0x6b, 0x34, 0x93, 0x69, 0xda, 0x30, 0xab}, /* Test 2 */
		{0x40, 0xa9, 0x45, 0x09, 0x23, 0x86, 0x64, 0xf2, 
		 0x44, 0xff, 0x8e, 0x3d, 0x13, 0xb1, 0x19, 0xd3}  /* Test 3 */
	};

	uint8_t pin[3][16] = {
		{0xd5, 0xa5, 0x10, 0x83, 0xa0, 0x4a, 0x19, 0x71, 
		 0xf1, 0x86, 0x49, 0xea, 0x8b, 0x79, 0x31, 0x1a}, /* Test 1 */
		{0x78, 0x85, 0xb5, 0x15, 0xe8, 0x4b, 0x1f, 0x08, 
		 0x2c, 0xc4, 0x99, 0x97, 0x6f, 0x17, 0x25, 0xce}, /* Test 2 */
		{0x1c, 0xe4, 0x48, 0x39, 0xba, 0xdd, 0xe3, 0x03, 
		 0x96, 0xd0, 0x3c, 0x4c, 0x36, 0xf2, 0x30, 0x06}  /* Test 3 */
	};

	uint8_t ka[3][16] = {
		{0x53, 0x9e, 0x4f, 0x27, 0x32, 0xe5, 0xae, 0x2d, 
		 0xe1, 0xe0, 0x40, 0x1f, 0x08, 0x13, 0xbd, 0x0d}, /* Test 1 */
		{0x04, 0x43, 0x57, 0x71, 0xe0, 0x3a, 0x9d, 0xac, 
		 0xeb, 0x8b, 0xb1, 0xa4, 0x93, 0xee, 0x9b, 0xd8}, /* Test 22*/
		{0x9c, 0xde, 0x4b, 0x60, 0xf9, 0xb5, 0x86, 0x1e, 
		 0xd9, 0xdf, 0x80, 0x85, 0x8b, 0xac, 0x6f, 0x7f}  /* Test 3 */
	};

	for (i = 0; i < testcount; i++) {

		printf("Testing K_master case %d of %d ... ", i + 1, testcount);

		memset(out_kmaster, 0, sizeof(out_kmaster));

		if (gen_kmaster(pin[i], rand[i], out_kmaster) != 0) {
			printf("error from gen_kmaster() ");
		}

		if (memcmp(out_kmaster, ka[i], 16) == 0) {
			printf("successful.\n");
		} else {
			printf("failed.\n");
			errorcount++;
		}
	}
	return errorcount;
}



int test_kinit()
{

	int i, testcount = 16, errorcount=0;
	uint8_t out_kinit[16];

	/* testing data taken from Bluetooth 2.0 specification */
	uint8_t rand[16][16] = {
		{0x24, 0xb1, 0x01, 0xfd, 0x56, 0x11, 0x7d, 0x42,
		 0xc0, 0x54, 0x5a, 0x42, 0x47, 0x35, 0x70, 0x48}, /* Test 1 */
		{0x32, 0x19, 0x64, 0x06, 0x1a, 0xc4, 0x9a, 0x43,
		 0x6f, 0x9f, 0xb9, 0x82, 0x4a, 0xc6, 0x3f, 0x8b}, /* Test 2  */
		{0xd4, 0xae, 0x20, 0xc8, 0x00, 0x94, 0x54, 0x7d,
		 0x70, 0x51, 0x93, 0x1b, 0x5c, 0xc2, 0xa8, 0xd6}, /* Test 3  */
		{0x27, 0x2b, 0x73, 0xa2, 0xe4, 0x0d, 0xb5, 0x2a,
		 0x6a, 0x61, 0xc6, 0x52, 0x05, 0x49, 0x79, 0x4a}, /* Test 4  */
		{0x7e, 0xdb, 0x65, 0xf0, 0x1a, 0x2f, 0x45, 0xa2,
		 0xbc, 0x9b, 0x24, 0xfb, 0x33, 0x90, 0x66, 0x7e}, /* Test 5  */
		{0x26, 0xa9, 0x23, 0x58, 0x29, 0x4d, 0xce, 0x97,
		 0xb1, 0xd7, 0x9e, 0xc3, 0x2a, 0x67, 0xe8, 0x1a}, /* Test 6  */
		{0x0e, 0xde, 0xf0, 0x53, 0x27, 0xea, 0xb5, 0x26,
		 0x24, 0x30, 0xf2, 0x1f, 0xc9, 0x1c, 0xe6, 0x82}, /* Test 7  */
		{0x86, 0x29, 0x0e, 0x28, 0x92, 0xf2, 0x78, 0xff,
		 0x6c, 0x3f, 0xb9, 0x17, 0xb0, 0x20, 0x57, 0x6a}, /* Test 8  */
		{0x3a, 0xb5, 0x2a, 0x65, 0xbb, 0x3b, 0x24, 0xa0,
		 0x8e, 0xb6, 0xcd, 0x28, 0x4b, 0x4b, 0x9d, 0x4b}, /* Test 9  */
		{0xa6, 0xdc, 0x44, 0x7f, 0xf0, 0x8d, 0x4b, 0x36,
		 0x6f, 0xf9, 0x6e, 0x6c, 0xf2, 0x07, 0xe1, 0x79}, /* Test 10 */
		{0x33, 0x48, 0x47, 0x0a, 0x7e, 0xa6, 0xcc, 0x6e,
		 0xb8, 0x1b, 0x40, 0x47, 0x21, 0x33, 0x26, 0x2c}, /* Test 11 */
		{0x0f, 0x5b, 0xb1, 0x50, 0xb4, 0x37, 0x1a, 0xe4,
		 0xe5, 0x78, 0x52, 0x93, 0xd2, 0x2b, 0x7b, 0x0c}, /* Test 12 */
		{0x14, 0x86, 0x62, 0xa4, 0xba, 0xa7, 0x3c, 0xfa,
		 0xdb, 0x55, 0x48, 0x91, 0x59, 0xe4, 0x76, 0xe1}, /* Test 13 */
		{0x19, 0x3a, 0x1b, 0x84, 0x37, 0x6c, 0x88, 0x88,
		 0x2c, 0x8d, 0x3b, 0x4e, 0xe9, 0x3b, 0xa8, 0xd5}, /* Test 14 */
		{0x14, 0x53, 0xdb, 0x4d, 0x05, 0x76, 0x54, 0xe8,
		 0xeb, 0x62, 0xd7, 0xd6, 0x2e, 0xc3, 0x60, 0x8c}, /* Test 15 */
		{0x13, 0x13, 0xf7, 0x11, 0x5a, 0x9d, 0xb8, 0x42,
		 0xfc, 0xed, 0xc4, 0xb1, 0x00, 0x88, 0xb4, 0x8d} /* Test 16 */
	};

	uint8_t pinlen[16] = {
		16, 15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1
	};

	uint8_t pin[16][16] = {
		{0xfd, 0x39, 0x7c, 0x7f, 0x5c, 0x1f, 0x93, 0x7c,
		 0xdf, 0x82, 0xd8, 0x81, 0x6c, 0xc3, 0x77, 0xe2}, /* Test  1 */
		{0xad, 0x95, 0x5d, 0x58, 0xb6, 0xb8, 0x85, 0x78,
		 0x20, 0xac, 0x12, 0x62, 0xd6, 0x17, 0xa6},       /* Test  2 */
		{0xe1, 0x23, 0x2e, 0x2c, 0x5f, 0x3b, 0x83, 0x3b,
		 0x33, 0x09, 0x08, 0x8a, 0x87, 0xb6},             /* Test  3 */
		{0x54, 0x9f, 0x26, 0x94, 0xf3, 0x53, 0xf5, 0x14,
		 0x57, 0x72, 0xd8, 0xae, 0x1e},                   /* Test  4 */
		{0x2e, 0x5a, 0x42, 0x79, 0x79, 0x58, 0x55, 0x7b,
		 0x23, 0x44, 0x7c, 0xa8},                         /* Test  5 */
		{0x05, 0xfb, 0xad, 0x03, 0xf5, 0x2f, 0xa9, 0x32,
		 0x4f, 0x77, 0x32},                               /* Test  6 */
		{0x82, 0x10, 0xe4, 0x73, 0x90, 0xf3, 0xf4, 0x8c,
		 0x32, 0xb3},                                     /* Test  7 */
		{0x3d, 0xcd, 0xff, 0xcf, 0xd0, 0x86, 0x80, 0x21,
		 0x07},	                                          /* Test  8 */
		{0xd0, 0xfb, 0x9b, 0x68, 0x38, 0xd4, 0x64, 0xd8}, /* Test  9 */
		{0x9c, 0x57, 0xe1, 0x0b, 0x47, 0x66, 0xcc},       /* Test 10 */
		{0xfc, 0xad, 0x16, 0x9d, 0x72, 0x95},             /* Test 11 */
		{0xb1, 0x0d, 0x06, 0x8b, 0xca},                   /* Test 12 */
		{0xfb, 0x20, 0xf1, 0x77},                         /* Test 13 */
		{0xa1, 0x23, 0xb9},                               /* Test 14 */
		{0x3e, 0xaf},                                     /* Test 15 */
		{0x6d}	                                          /* Test 16 */
	};

	uint8_t bdaddr[16][6] = {
		{0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, /* Test  1 */
		{0x03, 0x14, 0xc0, 0x64, 0x25, 0x43}, /* Test  2 */
		{0xfa, 0xbe, 0xcc, 0x58, 0xe6, 0x09}, /* Test  3 */
		{0x20, 0x48, 0x76, 0x81, 0xeb, 0x9f}, /* Test  4 */
		{0x04, 0xf0, 0xd2, 0x73, 0x7f, 0x02}, /* Test  5 */
		{0xb9, 0xac, 0x07, 0x1f, 0x9d, 0x70}, /* Test  6 */
		{0x7a, 0x3c, 0xdf, 0xe3, 0x77, 0xd1}, /* Test  7 */
		{0x79, 0x1a, 0x6a, 0x2c, 0x5c, 0xc3}, /* Test  8 */
		{0x25, 0xa8, 0x68, 0xdb, 0x91, 0xab}, /* Test  9 */
		{0x54, 0xeb, 0xd9, 0x32, 0x8c, 0xb6}, /* Test 10 */
		{0x43, 0x0d, 0x57, 0x2f, 0x88, 0x42}, /* Test 11 */
		{0xb4, 0x47, 0x75, 0x19, 0x9f, 0x29}, /* Test 12 */
		{0xa6, 0x83, 0xbd, 0x0b, 0x18, 0x96}, /* Test 13 */
		{0x44, 0x59, 0xa4, 0x46, 0x10, 0xf6}, /* Test 14 */
		{0x41, 0x1f, 0xbb, 0xb5, 0x1d, 0x1e}, /* Test 15 */
		{0x00, 0x8a, 0xa9, 0xbe, 0x62, 0xd5}, /* Test 16 */
	};

	uint8_t ka[16][16] = {
		{0xa5, 0xf2, 0xad, 0xf3, 0x28, 0xe4, 0xe6, 0xa2,
		 0xb4, 0x2f, 0x19, 0xc8, 0xb7, 0x4b, 0xa8, 0x84}, /* Test  1 */
		{0xc0, 0xec, 0x1a, 0x56, 0x94, 0xe2, 0xb4, 0x8d,
		 0x54, 0x29, 0x79, 0x11, 0xe6, 0xc9, 0x8b, 0x8f}, /* Test  2 */
		{0xd7, 0xb3, 0x9b, 0xe1, 0x3e, 0x36, 0x92, 0xc6,
		 0x5b, 0x4a, 0x9e, 0x17, 0xa9, 0xc5, 0x5e, 0x17}, /* Test  3 */
		{0x9a, 0xc6, 0x43, 0x09, 0xa3, 0x7c, 0x25, 0xc3,
		 0xb4, 0xa5, 0x84, 0xfc, 0x00, 0x2a, 0x16, 0x18}, /* Test  4 */
		{0xd3, 0xaf, 0x4c, 0x81, 0xe3, 0xf4, 0x82, 0xf0,
		 0x62, 0x99, 0x9d, 0xee, 0x78, 0x82, 0xa7, 0x3b}, /* Test  5 */
		{0xbe, 0x87, 0xb4, 0x4d, 0x07, 0x9d, 0x45, 0xa0,
		 0x8a, 0x71, 0xd1, 0x52, 0x08, 0xc5, 0xcb, 0x50}, /* Test  6 */
		{0xbf, 0x07, 0x06, 0xd7, 0x6e, 0xc3, 0xb1, 0x1c,
		 0xce, 0x72, 0x4b, 0x31, 0x1b, 0xf7, 0x1f, 0xf5}, /* Test  7 */
		{0xcd, 0xb0, 0xcc, 0x68, 0xf6, 0xf6, 0xfb, 0xd7,
		 0x0b, 0x46, 0x65, 0x2d, 0xe3, 0xef, 0x3f, 0xfb}, /* Test  8 */
		{0x98, 0x32, 0x18, 0x71, 0x8c, 0xa9, 0xaa, 0x97,
		 0x89, 0x2e, 0x31, 0x2d, 0x86, 0xdd, 0x95, 0x16}, /* Test  9 */
		{0x9c, 0xd6, 0x65, 0x0e, 0xad, 0x86, 0x32, 0x3e,
		 0x87, 0xca, 0xfb, 0x1f, 0xf5, 0x16, 0xd1, 0xe0}, /* Test 10 */
		{0x98, 0xf1, 0x54, 0x3a, 0xb4, 0xd8, 0x7b, 0xd5,
		 0xef, 0x52, 0x96, 0xfb, 0x5e, 0x3d, 0x3a, 0x21}, /* Test 11 */
		{0xc5, 0x50, 0x70, 0xb7, 0x2b, 0xc9, 0x82, 0xad,
		 0xb9, 0x72, 0xed, 0x05, 0xd1, 0xa7, 0x4d, 0xdb}, /* Test 12 */
		{0x7e, 0xc8, 0x64, 0xdf, 0x2f, 0x16, 0x37, 0xc7,
		 0xe8, 0x1f, 0x23, 0x19, 0xae, 0x8f, 0x46, 0x71}, /* Test 13 */
		{0xac, 0x0d, 0xaa, 0xbf, 0x17, 0x73, 0x2f, 0x63,
		 0x2e, 0x34, 0xef, 0x19, 0x36, 0x58, 0xbf, 0x5d}, /* Test 14 */
		{0x16, 0x74, 0xf9, 0xdc, 0x20, 0x63, 0xcc, 0x2b,
		 0x83, 0xd3, 0xef, 0x8b, 0xa6, 0x92, 0xeb, 0xef}, /* Test 15 */
		{0x38, 0xec, 0x02, 0x58, 0x13, 0x4e, 0xc3, 0xf0,
		 0x84, 0x61, 0xae, 0x5c, 0x32, 0x89, 0x68, 0xa1}  /* Test 16 */
	};

	for (i = 0; i < testcount; i++) {

		printf("Testing K_init case %d of %d ... ", i + 1, testcount);

		memset(out_kinit, 0, sizeof(out_kinit));

		if (gen_kinit(pinlen[i], pin[i], bdaddr[i], rand[i], 
			out_kinit) != 0) {
			printf("error from gen_kinit() ");
		}

		if (memcmp(out_kinit, ka[i], 16) == 0) {
			printf("successful.\n");
		} else {
			printf("failed.\n");
			errorcount++;
		}
	}
	return errorcount;
}


int test_e1()
{

	int testcount = 4, i, errorcount=0;
	uint8_t out_sres[4];
	uint8_t out_aco[12];

	uint8_t rand[4][16] = {
		{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, /* Test 1 */
		{0xbc, 0x3f, 0x30, 0x68, 0x96, 0x47, 0xc8, 0xd7,
		 0xc5, 0xa0, 0x3c, 0xa8, 0x0a, 0x91, 0xec, 0xeb}, /* Test 2 */
		{0x08, 0x91, 0xca, 0xee, 0x06, 0x3f, 0x5d, 0xa1,
		 0x80, 0x95, 0x77, 0xff, 0x94, 0xcc, 0xdc, 0xfb}, /* Test 3 */
		{0x0e, 0xcd, 0x61, 0x78, 0x2b, 0x41, 0x28, 0x48,
		 0x0c, 0x05, 0xdc, 0x45, 0x54, 0x2b, 0x1b, 0x8c} /* Test 4 */
	};

	uint8_t bdaddr[4][6] = {
		{0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, /* Test 1 */
		{0x7c, 0xa8, 0x9b, 0x23, 0x3c, 0x2d}, /* Test 2 */
		{0xc6, 0x2f, 0x19, 0xf6, 0xce, 0x98}, /* Test 3 */
		{0xf4, 0x28, 0xf0, 0xe6, 0x24, 0xb3} /* Test 4 */
	};

	uint8_t kab[4][16] = {
		{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, /* Test 1 */
		{0x15, 0x9d, 0xd9, 0xf4, 0x3f, 0xc3, 0xd3, 0x28,
		 0xef, 0xba, 0x0c, 0xd8, 0xa8, 0x61, 0xfa, 0x57}, /* Test 2 */
		{0x45, 0x29, 0x8d, 0x06, 0xe4, 0x6b, 0xac, 0x21,
		 0x42, 0x1d, 0xdf, 0xbe, 0xd9, 0x4c, 0x03, 0x2b}, /* Test 3 */
		{0x35, 0x94, 0x9a, 0x91, 0x42, 0x25, 0xfa, 0xba,
		 0xd9, 0x19, 0x95, 0xd2, 0x26, 0xde, 0x1d, 0x92} /* Test 4 */
	};

	uint8_t sres[4][4] = {
		{0x05, 0x6c, 0x0f, 0xe6}, /* Test 1 */
		{0x8d, 0x52, 0x05, 0xc5}, /* Test 2 */
		{0x00, 0x50, 0x7e, 0x5f}, /* Test 3 */
		{0x80, 0xe5, 0x62, 0x9c} /* Test 4 */
	};

	uint8_t aco[4][12] = {
		{0x48, 0xaf, 0xcd, 0xd4, 0xbd, 0x40,
		 0xfe, 0xf7, 0x66, 0x93, 0xb1, 0x13}, /* Test 1 */
		{0x3e, 0xd7, 0x5d, 0xf4, 0xab, 0xd9,
		 0xaf, 0x63, 0x8d, 0x14, 0x4e, 0x94}, /* Test 2 */
		{0x2a, 0x5f, 0x19, 0xfb, 0xf6, 0x09,
		 0x07, 0xe6, 0x9f, 0x39, 0xca, 0x9f}, /* Test 3 */
		{0xa6, 0xfe, 0x4d, 0xcd, 0xe3, 0x92,
		 0x46, 0x11, 0xd3, 0xcc, 0x6b, 0xa1}, /* Test 4 */
	};

	for (i = 0; i < testcount; i++) {

		printf("Testing E1 case %d ... ", i + 1);

		memset(out_sres, 0, sizeof(out_sres));
		memset(out_aco, 0, sizeof(out_aco));

		if (e1(kab[i], rand[i], bdaddr[i], out_sres, out_aco) != 0) {
			printf("Error from e1()\n");
		}

		if ((memcmp(out_aco, aco[i], sizeof(aco[i])) == 0) &&
			(memcmp(out_sres, sres[i], sizeof(sres[i])) == 0)) {
			printf("successful.\n");
		} else {
			printf("failed.\n");
			errorcount++;
		}
	}
	return errorcount;
}


int main()
{
	int errorcount=0, errortotal=0;
	
	printf("\nTesting E21 data ...\n");
	errorcount=test_e21();
	printf("Testing E21 complete, %d errors reported.\n", errorcount);
	errortotal += errorcount;
	errorcount=0;
		
	printf("\nTesting K_init data ...\n");
	errorcount=test_kinit();
	printf("Testing K_init complete, %d errors reported.\n", errorcount);
	errortotal += errorcount;
	errorcount=0;

	printf("\nTesting K_master data ...\n");
	errorcount=test_kmaster();
	printf("Testing K_master complete, %d errors reported.\n", errorcount);
	errortotal += errorcount;
	errorcount=0;

	printf("\nTesting E1 data ...\n");
	errorcount=test_e1();
	printf("Testing E1 complete, %d errors reported.\n", errorcount);
	errortotal += errorcount;

	printf("\nTesting complete, %d total errors reported.\n", errortotal);

	return errortotal;
}
